package tv.mudu.mdpusher_plugin;

import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Handler;
import android.util.Log;
import android.view.SurfaceView;


import com.alivc.component.custom.AlivcLivePushCustomFilter;
import com.alivc.live.filter.TaoBeautyFilter;
import com.alivc.live.pusher.AlivcBeautyLevelEnum;
import com.alivc.live.pusher.AlivcFpsEnum;
import com.alivc.live.pusher.AlivcLivePushCameraTypeEnum;
import com.alivc.live.pusher.AlivcLivePushConfig;
import com.alivc.live.pusher.AlivcLivePushError;
import com.alivc.live.pusher.AlivcLivePushErrorListener;
import com.alivc.live.pusher.AlivcLivePushInfoListener;
import com.alivc.live.pusher.AlivcLivePushNetworkListener;
import com.alivc.live.pusher.AlivcLivePusher;
import com.alivc.live.pusher.AlivcPreviewOrientationEnum;
import com.alivc.live.pusher.AlivcResolutionEnum;

import java.util.HashMap;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;
import io.flutter.plugin.common.StandardMessageCodec;

/** MdpusherPlugin */
public class MdpusherPlugin implements MethodCallHandler {
  static private MethodChannel mChannel;
  static private Registrar mRegistrar;
  static private MdViewFactory viewFactory;
  private boolean portrait;
  private Integer frameRate;
  private Integer bitrate;
  private Integer resolution;// 0:480P 1:540P 2:720P
  private boolean frontCamera;
  private boolean pushing;
  private boolean beautyOn;
  private boolean flashLightOn;
  private boolean micOn;
  private String url;
  private boolean isPreviewed;
  private AlivcLivePusher mLivePusher;
  private AlivcLivePushConfig mLivePushConfig;
  private TaoBeautyFilter taoBeautyFilter;
  Map<String, AlivcFpsEnum> frameMap = new HashMap<String, AlivcFpsEnum>(){
    {
      put("8", AlivcFpsEnum.FPS_8);
      put("10", AlivcFpsEnum.FPS_10);
      put("12", AlivcFpsEnum.FPS_12);
      put("15", AlivcFpsEnum.FPS_15);
      put("20", AlivcFpsEnum.FPS_20);
      put("25", AlivcFpsEnum.FPS_25);
      put("30", AlivcFpsEnum.FPS_30);
    }
  };

  /** Plugin registration. */
  public static void registerWith(Registrar registrar) {
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "mdpusher_plugin");
    channel.setMethodCallHandler(new MdpusherPlugin());
    mChannel = channel;
    mRegistrar = registrar;
    viewFactory = new MdViewFactory(StandardMessageCodec.INSTANCE);
    registrar.platformViewRegistry().registerViewFactory("mdview",viewFactory);

  }

  @Override
  public void onMethodCall(MethodCall call, Result result) {
    Log.d("MDPusher","method:"+call.method+"\nargs:"+(call.arguments == null?"null":call.arguments.toString()));
    if (call.method.equals("config")){
      if (!pushing) {
        portrait = Boolean.valueOf(call.argument("portrait").toString());
        frameRate = Integer.valueOf(call.argument("frameRate").toString());
        bitrate = Integer.valueOf(call.argument("bitrate").toString());
        frontCamera = Boolean.valueOf(call.argument("frontCamera").toString());
        micOn = Boolean.valueOf(call.argument("micOn").toString());
        beautyOn = Boolean.valueOf(call.argument("beautyOn").toString());
        resolution = Integer.valueOf(call.argument("resolution").toString());
        initPusher();
      }
    } else if (call.method.equals("startPreview")) {
      int viewId = Integer.valueOf(call.argument("viewId").toString());
      startPreview(viewId);
    } else if (call.method.equals("stopPreview")) {
      isPreviewed = false;
      mLivePusher.stopPreview();
    } else if (call.method.equals("startPush")) {
      url = call.argument("url");
      startVideoStream(url);
    } else if (call.method.equals("stopPush")) {
      if (pushing) {
        try {
          mLivePusher.stopPush();
          pushing = false;
        } catch (Exception e) {
          Log.d("MDPusher", e.toString());
        }
      }
    } else if (call.method.equals("switchCamera")) {
      frontCamera = !frontCamera;
      mLivePusher.switchCamera();
      if (!frontCamera && flashLightOn) {
        setFlashLightOn(false);
      }
    } else if (call.method.equals("setFlashLight")) {
      setFlashLightOn(Boolean.valueOf(call.argument("flashLight").toString()));
    } else if (call.method.equals("setBeautyOn")) {
      beautyOn = Boolean.valueOf(call.argument("beautyOn").toString());
      mLivePusher.setBeautyOn(beautyOn);
    } else if (call.method.equals("setMicOn")) {
      micOn = Boolean.valueOf(call.argument("micOn").toString());
      mLivePusher.setMute(!micOn);
    } else if (call.method.equals("destory")) {
      if(mLivePusher != null) {
        try {
          mLivePusher.destroy();
        } catch (IllegalStateException e) {
          e.printStackTrace();
        }
      }
      mLivePusher = null;
    } else {
      result.notImplemented();
      return;
    }
    result.success(null);
  }

  private void initPusher() {
      if (mLivePusher == null) {
          mLivePusher = new AlivcLivePusher();
      }
      if (mLivePushConfig == null) {
          mLivePushConfig = new AlivcLivePushConfig();
      }

    mLivePushConfig.setResolution(mapConfigReslution());

    mLivePushConfig.setPreviewOrientation(mapConfigOrientation());

    mLivePushConfig.setFps(mapConfigFps());

    mLivePushConfig.setBeautyLevel(AlivcBeautyLevelEnum.BEAUTY_Normal);
    mLivePushConfig.setBeautyRuddy(40);
    mLivePushConfig.setBeautyBuffing(40);
    mLivePushConfig.setBeautyWhite(70);

    mLivePushConfig.setTargetVideoBitrate(bitrate);
    mLivePushConfig.setInitialVideoBitrate(bitrate);
    mLivePushConfig.setMinVideoBitrate((int) Math.round(bitrate * 0.5));

    mLivePushConfig.setExposure(20);
    mLivePushConfig.setCameraType(mapConfigCameraType());

    Log.d("MDPusher", "fps:"+mapConfigFps()+" - bitrate:"+bitrate+" - reslution:"+mapConfigReslution());

    mLivePusher.init(mRegistrar.activeContext(), mLivePushConfig);

    mLivePusher.setLivePushInfoListener(mPushInfoListener);
    mLivePusher.setLivePushNetworkListener(mPushNetworkListener);
    mLivePusher.setLivePushErrorListener(mPushErrorListener);

    mLivePusher.setCustomFilter(new AlivcLivePushCustomFilter() {
      @Override
      public void customFilterCreate() {
        taoBeautyFilter = new TaoBeautyFilter();
        taoBeautyFilter.customFilterCreate();
      }

      @Override
      public void customFilterUpdateParam(float fSkinSmooth, float fWhiten, float fWholeFacePink, float fThinFaceHorizontal, float fCheekPink, float fShortenFaceVertical, float fBigEye) {
        if (taoBeautyFilter != null) {
          taoBeautyFilter.customFilterUpdateParam(fSkinSmooth, fWhiten, fWholeFacePink, fThinFaceHorizontal, fCheekPink, fShortenFaceVertical, fBigEye);
        }
      }

      @Override
      public void customFilterSwitch(boolean on)
      {
        if(taoBeautyFilter != null) {
          taoBeautyFilter.customFilterSwitch(on);
        }
      }

      @Override
      public int customFilterProcess(int inputTexture, int textureWidth, int textureHeight, long extra) {
        if (taoBeautyFilter != null) {
          return taoBeautyFilter.customFilterProcess(inputTexture, textureWidth, textureHeight, extra);
        }
        return inputTexture;
      }

      @Override
      public void customFilterDestroy() {
        if (taoBeautyFilter != null) {
          taoBeautyFilter.customFilterDestroy();
        }
        taoBeautyFilter = null;
      }
    });
  }

  private void startPreview(final int viewId) {
    new Handler().postDelayed(new Runnable() {
      @Override
      public void run() {
        SurfaceView surfaceView = viewFactory.getSurfaceViewById(viewId);
        if (surfaceView != null) {
          mLivePusher.startPreview(surfaceView);
        }
      }
    },100);
  }

  private void startVideoStream(String url){
    String addr="rtmp://video.mudu.tv/watch/"+url;
    if(url.contains("rtmp://")){
      addr=url;
    }
    if (!isNetworkConnected(mRegistrar.activeContext())) {
//      customToast("网络断开", 0);
      return ;
    }
    try {
      mLivePusher.startPush(addr);
    } catch (Exception e) {
      Log.d("MDPusher", e.toString() + " -- " + e.getMessage() + " -- " + e.getCause());
      try {
        mLivePusher.reconnectPushAsync(addr);
      } catch (Exception ee) {
        Log.d("MDPusher", ee.toString());
      }
    }

  }


  public void setFlashLightOn(boolean flashLightOn) {
    if (frontCamera) return;;
    this.flashLightOn = flashLightOn;
    mLivePusher.setFlash(flashLightOn);
  }

  /**
   * 自定义方法
   */
  //获取分辨率
  private AlivcResolutionEnum mapConfigReslution() {
    switch (resolution) {
      case 0:
        return AlivcResolutionEnum.RESOLUTION_480P;
      case 1:
        return AlivcResolutionEnum.RESOLUTION_540P;
      case 2:
        return AlivcResolutionEnum.RESOLUTION_720P;
      default:
        return AlivcResolutionEnum.RESOLUTION_480P;
    }
  }

  //获得推流方向
  private AlivcPreviewOrientationEnum mapConfigOrientation() {
    if (portrait) {
      return AlivcPreviewOrientationEnum.ORIENTATION_PORTRAIT;
    }
    return AlivcPreviewOrientationEnum.ORIENTATION_LANDSCAPE_HOME_LEFT;
  }

  //获取fps
  private AlivcFpsEnum mapConfigFps() {
    Log.d("MDPusher", "fps:::" + frameRate);
    AlivcFpsEnum resultFps = AlivcFpsEnum.FPS_25;
    for (String key : frameMap.keySet()) {
      if (Integer.valueOf(key).equals(frameRate)) {
        resultFps = frameMap.get(key);
        break;
      }
    }
    Log.d("MDPusher", "fps:::::::" + resultFps);
    return resultFps;
  }
  //获取摄像头方向
  private AlivcLivePushCameraTypeEnum mapConfigCameraType() {
    if (frontCamera) {
      return AlivcLivePushCameraTypeEnum.CAMERA_TYPE_FRONT;
    } else {
      return AlivcLivePushCameraTypeEnum.CAMERA_TYPE_BACK;
    }
  }

  private AlivcLivePushInfoListener mPushInfoListener = new AlivcLivePushInfoListener() {
    @Override
    public void onPreviewStarted(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onPreviewStarted");
      isPreviewed = true;
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onPreviewStarted",null);
        }
      });
    }

    @Override
    public void onPreviewStoped(AlivcLivePusher alivcLivePusher) {
      isPreviewed = false;
      Log.d("MDPusher", "onPreviewStoped");
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onPreviewStoped",null);
        }
      });
    }

    @Override
    public void onPushStarted(AlivcLivePusher alivcLivePusher) {
      pushing = true;
      Log.d("MDPusher", "onPushStarted");
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onPushStarted",null);
        }
      });
    }

    @Override
    public void onPushPauesed(AlivcLivePusher alivcLivePusher) {

    }

    @Override
    public void onPushResumed(AlivcLivePusher alivcLivePusher) {

    }

    @Override
    public void onPushStoped(AlivcLivePusher alivcLivePusher) {
      pushing = false;
      Log.d("MDPusher", "onPushStoped");
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onPushStoped",null);
        }
      });
    }

    @Override
    public void onPushRestarted(AlivcLivePusher alivcLivePusher) {

    }

    @Override
    public void onFirstFramePreviewed(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onFirstFramePreviewed");
    }

    @Override
    public void onFirstAVFramePushed(AlivcLivePusher alivcLivePusher) {

    }

    @Override
    public void onDropFrame(AlivcLivePusher alivcLivePusher, int i, int i1) {
      Log.d("MDPusher", "掉帧");
    }

    @Override
    public void onAdjustBitRate(AlivcLivePusher alivcLivePusher, int i, int i1) {
      Log.d("MDPusher", "onAdjustBitRate");
    }

    @Override
    public void onAdjustFps(AlivcLivePusher alivcLivePusher, int i, int i1) {
      Log.d("MDPusher", "onAdjustFps");
    }
  };
  private AlivcLivePushNetworkListener mPushNetworkListener = new AlivcLivePushNetworkListener() {
    @Override
    public void onNetworkPoor(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "=== onNetworkBusy ===");
    }

    @Override
    public void onNetworkRecovery(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onNetworkRecovery");
    }

    @Override
    public void onReconnectStart(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onReconnectStart");
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onReconnectStart",null);
        }
      });
    }

    @Override
    public void onReconnectFail(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onReconnectFail " + mLivePusher.isPushing());
      pushing = false;
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onReconnectFail",null);
        }
      });
    }

    @Override
    public void onReconnectSucceed(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onReconnectSucceed");
      pushing = true;
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onReconnectSucceed",null);
        }
      });
    }

    @Override
    public void onSendDataTimeout(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onSendDataTimeout");
    }

    @Override
    public void onConnectionLost(AlivcLivePusher alivcLivePusher) {
      pushing = false;
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onConnectionLost",null);
        }
      });
    }

    @Override
    public void onPacketsLost(AlivcLivePusher alivcLivePusher) {

    }

    @Override
    public void onConnectFail(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onConnectFail " + mLivePusher.isPushing());
      pushing = false;
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onConnectFail",null);
        }
      });
    }

    @Override
    public String onPushURLAuthenticationOverdue(AlivcLivePusher alivcLivePusher) {
      Log.d("MDPusher", "onPushURLAuthenticationOverdue");
      return null;
    }

    @Override
    public void onSendMessage(AlivcLivePusher alivcLivePusher) {

    }
  };
  private AlivcLivePushErrorListener mPushErrorListener = new AlivcLivePushErrorListener() {
    @Override
    public void onSystemError(AlivcLivePusher alivcLivePusher, AlivcLivePushError alivcLivePushError) {
      Log.d("MDPusher", "onSystemError");
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onSystemError",null);
        }
      });
    }

    @Override
    public void onSDKError(AlivcLivePusher alivcLivePusher, AlivcLivePushError alivcLivePushError) {
      Log.d("MDPusher", "onSdkError");
      pushing = false;
      isPreviewed = false;
      mRegistrar.activity().runOnUiThread(new Runnable() {
        @Override
        public void run() {
          mChannel.invokeMethod("onSDKError",null);
        }
      });
    }
  };

  public boolean isNetworkConnected(Context context) {
    if (context != null) {
      ConnectivityManager mConnectivityManager = (ConnectivityManager) context
              .getSystemService(Context.CONNECTIVITY_SERVICE);
      NetworkInfo mNetworkInfo = mConnectivityManager.getActiveNetworkInfo();
      if (mNetworkInfo != null) {
        return mNetworkInfo.isAvailable();
      }
    }
    return false;
  }
}
