#import "MdpusherPlugin.h"

#import "MDViewFactory.h"

#import <AlivcLivePusher/AlivcLivePusher.h>
#import <AlivcLibBeauty/AlivcLibBeautyManager.h>

static NSObject<FlutterPluginRegistrar> *mRegistrar;
static FlutterMethodChannel *mChannel;
static MDViewFactory *mFactory;

@interface MdpusherPlugin ()<AlivcLivePusherInfoDelegate,AlivcLivePusherErrorDelegate,AlivcLivePusherNetworkDelegate,AlivcLivePusherCustomFilterDelegate>
@property (nonatomic, assign) BOOL pushing;
@property (nonatomic, assign) BOOL portrait;
@property (nonatomic, assign) BOOL frontCamera;
@property (nonatomic, assign) BOOL beautyOn;
@property (nonatomic, assign) BOOL micOn;
@property (nonatomic, assign) BOOL flashLightOn;
@property (nonatomic, assign) NSInteger frameRate;
@property (nonatomic, assign) NSInteger bitrate;
@property (nonatomic, assign) NSInteger resolution;
@property (nonatomic, copy) NSString *url;
@property (nonatomic, strong) AlivcLivePusher *livePusher;
@property (nonatomic, strong) AlivcLivePushConfig *livePushConfig;
@property (nonatomic, assign) int lastOrientation;
@end

@implementation MdpusherPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"mdpusher_plugin"
            binaryMessenger:[registrar messenger]];
  mRegistrar = registrar;
  mChannel = channel;
  mFactory = [[MDViewFactory alloc]init];
  [registrar registerViewFactory:mFactory withId:@"mdview"];
  MdpusherPlugin* instance = [[MdpusherPlugin alloc] init];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([call.method isEqualToString:@"config"]) {
      if(!_pushing){
          _portrait = [[call.arguments objectForKey:@"portrait"] boolValue];
          _frameRate = [[call.arguments objectForKey:@"frameRate"] integerValue];
          _bitrate = [[call.arguments objectForKey:@"bitrate"] integerValue];
          _frontCamera = [[call.arguments objectForKey:@"frontCamera"] boolValue];
          _micOn = [[call.arguments objectForKey:@"micOn"] boolValue];
          _beautyOn = [[call.arguments objectForKey:@"beautyOn"] boolValue];
          _resolution = [[call.arguments objectForKey:@"resolution"] integerValue];
          [self initPusher];
      }
  } else if ([call.method isEqualToString:@"startPreview"]) {
      int viewId = [[call.arguments objectForKey:@"viewId"] integerValue];
      dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
          [self startPreview:viewId];
      });
  } else if ([call.method isEqualToString:@"stopPreview"]) {
      [_livePusher stopPreview];
  } else if ([call.method isEqualToString:@"startPush"]) {
      NSString *url = [call.arguments objectForKey:@"url"];
      int ret = [_livePusher startPushWithURLAsync:url];
      if (ret) {
          
      }
  } else if ([call.method isEqualToString:@"stopPush"]) {
      if (_pushing) {
          int ret = [_livePusher stopPush];
          if (ret) {
              
          }
      }
  } else if ([call.method isEqualToString:@"switchCamera"]) {
      int ret = [_livePusher switchCamera];
      if (!ret) {
          _frontCamera = !_frontCamera;
          if (!_frontCamera && _flashLightOn) {
              [self changeFlashLight:NO];
          }
      }
  } else if ([call.method isEqualToString:@"setFlashLight"]) {
      _flashLightOn = [[call.arguments objectForKey:@"flashLight"] boolValue];
      [self changeFlashLight:_flashLightOn];
  } else if ([call.method isEqualToString:@"setBeautyOn"]) {
      _beautyOn = [[call.arguments objectForKey:@"beautyOn"] boolValue];
      [_livePusher setBeautyOn:_beautyOn];
  } else if ([call.method isEqualToString:@"setMicOn"]) {
      _micOn = [[call.arguments objectForKey:@"micOn"] boolValue];
      [_livePusher setMute:_micOn];
  } else if ([call.method isEqualToString:@"destory"]) {
      [_livePusher destory];
  } else if ([call.method isEqualToString:@"changeOrientation"]) {
      int orientationVal;
      if (call.arguments && [call.arguments objectForKey:@"orientation"]) {
          orientationVal = [[call.arguments objectForKey:@"orientation"] intValue];
          UIDeviceOrientation deviceOrientation = [[UIDevice currentDevice]orientation];
          if (deviceOrientation == UIDeviceOrientationPortrait) {
              _lastOrientation = 0;
          } else {
              _lastOrientation = 1;
          }
      } else {
          orientationVal = _lastOrientation;
      }
      if (orientationVal == 0) {
          [[UIDevice currentDevice]setValue:@(UIDeviceOrientationPortrait) forKey:@"orientation"];
      } else {
          [[UIDevice currentDevice]setValue:@(UIDeviceOrientationLandscapeLeft) forKey:@"orientation"];
      }
  } else {
    result(FlutterMethodNotImplemented);
    return;
  }
  result(nil);
}

- (void)initPusher {
    if (_livePushConfig == nil) {
        _livePushConfig = [[AlivcLivePushConfig alloc]init];
    }
    
    _livePushConfig.qualityMode = AlivcLivePushQualityModeCustom;
    _livePushConfig.minVideoBitrate = _bitrate;
    _livePushConfig.targetVideoBitrate = _bitrate;
    _livePushConfig.initialVideoBitrate = _bitrate;
    if (_resolution == 0) {
        _livePushConfig.resolution = AlivcLivePushResolution480P;
    } else if (_resolution == 1) {
        _livePushConfig.resolution = AlivcLivePushResolution540P;
    } else {
        _livePushConfig.resolution = AlivcLivePushResolution720P;
    }
    _livePushConfig.fps = _frameRate;
    _livePushConfig.minFps = _frameRate;
    if (_portrait) {
        _livePushConfig.orientation = AlivcLivePushOrientationPortrait;
    } else {
        _livePushConfig.orientation = AlivcLivePushOrientationLandscapeRight;
    }
    _livePushConfig.pushMirror = NO;
    if (_frontCamera) {
        _livePushConfig.cameraType = AlivcLivePushCameraTypeFront;
    } else {
        _livePushConfig.cameraType = AlivcLivePushCameraTypeBack;
    }
    
    if (_livePusher) {
        [_livePusher destory];
    }
    _livePusher = [[AlivcLivePusher alloc]initWithConfig:_livePushConfig];
    [_livePusher setCaptureVolume:0];
    [_livePusher setLogLevel:AlivcLivePushLogLevelFatal];
    [_livePusher setInfoDelegate:self];
    [_livePusher setErrorDelegate:self];
    [_livePusher setNetworkDelegate:self];
    [_livePusher setCustomFilterDelegate:self];
}

- (void)startPreview:(int)viewId {
    UIView *rendererView = [mFactory rendererViewForId:viewId];
    int ret = [_livePusher startPreview:rendererView];
}

- (void)changeFlashLight:(BOOL)on {
    if (_frontCamera) {
        return;
    }
    int ret = [_livePusher setFlash:on];
    if (!ret) {
        _flashLightOn = on;
    }
}

#pragma mark- pusher info delegate
- (void)onPreviewStarted:(AlivcLivePusher *)pusher {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onPreviewStarted" arguments:nil];
    });
}
/**
 停止预览回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onPreviewStoped:(AlivcLivePusher *)pusher {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onPreviewStoped" arguments:nil];
    });
}


/**
 渲染第一帧回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onFirstFramePreviewed:(AlivcLivePusher *)pusher {
    
}


/**
 推流开始回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onPushStarted:(AlivcLivePusher *)pusher {
    _pushing = true;
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onPushStarted" arguments:nil];
    });
}


/**
 推流暂停回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onPushPauesed:(AlivcLivePusher *)pusher {
    
}


/**
 推流恢复回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onPushResumed:(AlivcLivePusher *)pusher {
    
}


/**
 重新推流回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onPushRestart:(AlivcLivePusher *)pusher {
    
}


/**
 推流停止回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onPushStoped:(AlivcLivePusher *)pusher {
    _pushing = NO;
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onPushStoped" arguments:nil];
    });
}

#pragma mark- pusher error delegate

/**
 系统错误回调
 
 @param pusher 推流AlivcLivePusher
 @param error error
 */
- (void)onSystemError:(AlivcLivePusher *)pusher error:(AlivcLivePushError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onSystemError" arguments:nil];
    });
}


/**
 SDK错误回调
 
 @param pusher 推流AlivcLivePusher
 @param error error
 */
- (void)onSDKError:(AlivcLivePusher *)pusher error:(AlivcLivePushError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onSDKError" arguments:nil];
    });
}

#pragma mark- pusher network delegate

/**
 网络差回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onNetworkPoor:(AlivcLivePusher *)pusher {
    
}


/**
 推流链接失败
 
 @param pusher 推流AlivcLivePusher
 @param error error
 */
- (void)onConnectFail:(AlivcLivePusher *)pusher error:(AlivcLivePushError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onConnectFail" arguments:nil];
    });
}


/**
 网络恢复
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onConnectRecovery:(AlivcLivePusher *)pusher {
    
}


/**
 重连开始回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onReconnectStart:(AlivcLivePusher *)pusher {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onReconnectStart" arguments:nil];
    });
}


/**
 重连成功回调
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onReconnectSuccess:(AlivcLivePusher *)pusher {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onReconnectSuccess" arguments:nil];
    });
}


/**
 重连失败回调
 
 @param pusher 推流AlivcLivePusher
 @param error error
 */
- (void)onReconnectError:(AlivcLivePusher *)pusher error:(AlivcLivePushError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        [mChannel invokeMethod:@"onReconnectFail" arguments:nil];
    });
}


/**
 发送数据超时
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onSendDataTimeout:(AlivcLivePusher *)pusher{
    
}


/**
 推流URL的鉴权时长即将过期(将在过期前1min内发送此回调)
 
 @param pusher 推流AlivcLivePusher
 @return 新的推流URL
 */
- (NSString *)onPushURLAuthenticationOverdue:(AlivcLivePusher *)pusher {
    return [[NSString alloc]init];
}


/**
 发送SEI Message 通知
 
 @param pusher 推流AlivcLivePusher
 */
- (void)onSendSeiMessage:(AlivcLivePusher *)pusher {
    
}

#pragma mark- pusher custom filter delegate

/**
 通知外置滤镜创建回调
 */
- (void)onCreate:(AlivcLivePusher *)pusher context:(void*)context {
    [[AlivcLibBeautyManager shareManager] create:context];
}
/**
 通知外置滤镜设置参数
 */
- (void)updateParam:(AlivcLivePusher *)pusher buffing:(float)buffing whiten:(float)whiten pink:(float)pink cheekpink:(float)cheekpink thinface:(float)thinface shortenface:(float)shortenface bigeye:(float)bigeye {
    [[AlivcLibBeautyManager shareManager] setParam:buffing
                                            whiten:whiten
                                              pink:pink
                                         cheekpink:cheekpink
                                          thinface:thinface
                                       shortenface:shortenface
                                            bigeye:bigeye];
}
/**
 通知外置滤镜开馆
 */
- (void)switchOn:(AlivcLivePusher *)pusher on:(bool)on {
    [[AlivcLibBeautyManager shareManager] switchOn:on];
}
/**
 通知外置滤镜处理回调
 */
- (int)onProcess:(AlivcLivePusher *)pusher texture:(int)texture textureWidth:(int)width textureHeight:(int)height extra:(long)extra {
    return [[AlivcLibBeautyManager shareManager] process:texture
                                                   width:width
                                                  height:height
                                                   extra:extra];;
}

/**
 通知外置滤镜销毁回调
 */
- (void)onDestory:(AlivcLivePusher *)pusher {
    [[AlivcLibBeautyManager shareManager] destroy];
}


@end
